# ergmR_explain.R



install.packages(c('statnet','sna' , 'latticeExtra','igraph', 'intergraph'))
#open libraries

library(statnet)
library(ergm)
library(sna)
library(latticeExtra)
library(igraph)
library(intergraph)
# set path to data
setwd("c:/temp/")# mind the "/"

# adolescent school network, read from the web server
#friends <- read.table("http://www.barkhof.uni-bremen.de/~mwindzio/fr3.dat") # friendship network
#explain <- read.table("http://www.barkhof.uni-bremen.de/~mwindzio/f1erklaere_ihm_matrix3.dat") #  explaining academic issues network
#attributes <- read.table("http://www.barkhof.uni-bremen.de/~mwindzio/attribute3.dat", header = TRUE) # actor attributes


friends <- read.table("fr3.dat") # friendship network
explain <- read.table("f1erklaere_ihm_matrix3.dat") #  explaining academic issues network
attributes <- read.table("attribute3.dat", header = TRUE) # actor attributes


# read additional network matrix: ego explains alter scholastic issues !
ncol(explain)
dim(explain)
head(explain)
explain <- explain[,3:111] # delete first two columns => matching variables we don't need
ncol(explain)
dim(explain)


# convert matrix into network matrix
friends <- network(friends,matrix.type="adjacency",directed=TRUE)
friends

# convert matrix into network matrix
friendsn <- network(friends,matrix.type="adjacency",directed=TRUE)
friendsn


explain <- network(explain,matrix.type="adjacency",directed=TRUE) # ego explains to alter
explain

# triad.census(asIgraph(explain))

plot(explain)

# merge attributes to network matrix, the %v% extracts a vector


explain %v% "alter" <- attributes$f2Alter
explain %v% "frau" <- attributes$frau
explain %v% "migrant" <- attributes$mh
explain
explain[,]
plot(explain)

list.vertex.attributes(explain)


friendsn %v% "alter" <- attributes$f2Alter
friendsn %v% "frau" <- attributes$frau
friendsn %v% "migrant" <- attributes$mh
friendsn


# descriptive analysis of possible association (homophily)
# each variable is cross-classified, row: ego; col: alter
# cells show numbers of ties
mixingmatrix(explain, "frau")
mixingmatrix(explain, "migrant")



# compare our network with a random network
# regarding Transitivity and reciprocity/mutuality
# this is a first motivation of ERGMs
den <- gden(explain)

# create random networks
random_graph<-network(109, directed=TRUE, density=den, seed=1234) # create random digraph
random_graph
plot(random_graph)
plot(explain)
plot(explain, displayisolates=FALSE)

# different spring-embedder-layout algorithms
gplot(explain, mode = "fruchtermanreingold")
gplot(explain, mode = "kamadakawai")

# holding layouts constant
# plotting the network: take the same layout for explain, random_graph and for friendship
l <- igraph::layout.kamada.kawai(   igraph::graph_from_adjacency_matrix(as.matrix(explain), mode = c("directed")  )   )

igraph.options(vertex.label=NA, vertex.size=8, vertex.color="red" , edge.arrow.size=0.1, edge.color="black")

plot(igraph::graph_from_adjacency_matrix(as.matrix(explain), mode = c("directed")  ), layout = l)
plot(igraph::graph_from_adjacency_matrix(as.matrix(friends), mode = c("directed")  ), layout = l)
plot(igraph::graph_from_adjacency_matrix(as.matrix(random_graph), mode = c("directed")  ), layout = l)


####################################################
# compare random network with empirical network ####
####################################################

# compare reciprocity
grecip(explain, measure="edgewise")
grecip(random_graph, measure="edgewise")

# compare Transitivity
gtrans(explain) 
gtrans(random_graph)

# this is important insight into how emprical social networks
# differ between random networks 


# dyad and triad census
dyad.census(asIgraph(explain))  	# M,A,N counts
dyad.census(asIgraph(random_graph))		# M,A,N counts
triad.census(asIgraph(explain,mode="digraph"))	# Directed triad census
triad.census(asIgraph(random_graph,mode="digraph"))	# Directed triad census

# TRIAD CENSUS
# 1.) no. of mutual dyads
# 2.) no. of asymmetric dyads
# 3.) no. of null-dyads
# 4. character of further distinction (transitive, cyclic, u=> up, d=>down)

network.edgecount(explain)
network.size(explain)



###########################
### ergm ##################
###########################

gden(explain)
# most simple model
model <- ergm(explain ~ edges, verbose=TRUE, control.ergm=c(seed=1234,MCMCsamplesize=1000))
summary(model)

exp(-3.78263) / (1+ exp(-3.78263))
gden(explain)

model0 <- ergm(explain ~ edges + mutual, verbose=TRUE, control.ergm=c(seed=1234,MCMCsamplesize=1000))
summary(model0)

model1 <- ergm(explain ~ edges + mutual + absdiff("alter") 
               + nodematch("frau") + nodematch("migrant") , 
               verbose=TRUE, control.ergm=c(seed=1234,MCMCsamplesize=1000))
summary(model1)

# more realistic model => not necessarily degenerate, but check diagnostics
model2 <- ergm(explain ~ edges + mutual + gwesp(decay=0.3, fixed=TRUE) + absdiff("alter") 
               + nodematch("frau") + nodematch("migrant") , 
               verbose=FALSE, control.ergm=c(seed=1234,MCMCsamplesize=1000))
summary(model2)

# check goodness of fit of model2 => not perfect!
model2.gof <- gof(model2 ~ idegree + odegree + distance + espartners )
plot(model2.gof)

# include dyadic covariate "friends"
model3 <- ergm(explain ~ edges + mutual + gwesp(decay=0.3, fixed=TRUE) + absdiff("alter") 
	+ nodematch("frau") + nodematch("migrant") + edgecov(friends), 
	verbose=FALSE, control.ergm=c(seed=1234,MCMCsamplesize=1000))
summary(model3)

# what if model did not converge? "init=model3$coef" uses previous estimates as starting values
model3 <- ergm(explain ~ edges + mutual + gwesp(decay=0.3, fixed=TRUE) + absdiff("alter") 
               + nodematch("frau") + nodematch("migrant") + edgecov(friends), 
               verbose=FALSE, control.ergm=c(seed=1234,MCMCsamplesize=1000, init=model3$coef))

# control the simulation here: nsim=200,  burnin = 10000, interval = 10000, seed = 567
model3.gof <- gof(model3 ~ idegree + odegree + distance + espartners, nsim=200, burnin = 10000, interval = 10000, seed = 567  )
plot(model3.gof)
# check the table
model3.gof

# plots the simulated network
sim1 <- simulate(model3) #Creates a simulated network from the fitted model 
plot(sim1,vertex.col="red" ) # plots the simulate network 


# check the MCMC traces hier 
mcmc.diagnostics(model3)


# ERGM of "friends"
model1f <- ergm(friendsn ~ edges + mutual + gwesp(decay=0.2, fixed=TRUE) + absdiff("alter") 
               + nodematch("frau") + nodematch("migrant"), 
               verbose=FALSE, control.ergm=c(seed=1234,MCMCsamplesize=1000))
summary(model1f)

