# intro_SNA.R, Michael Windzio
# short intro on social network analysis with R
# some parts are based on steven goodreau et al. (2008)



#pakete installieren
install.packages('igraph')
install.packages('statnet')
install.packages('sna')
install.packages('latticeExtra')


#statnet::update_statnet()

library(statnet)
#library(ergm)
library(sna)
library(latticeExtra)


install.packages('rgl', dependencies=TRUE)
library(rgl)


# set path to data
setwd("c:/temp/")# mind the "/"
#setwd("E:/lehre/aktuell/sose2016/Netzwerkanalyse/data")# mind the "/"
rm(list=ls())

#  read network matrix, adolescent school network, read from the web server
friends <- read.table("http://www.barkhof.uni-bremen.de/~mwindzio/fr3.dat") # friendship network
explain <- read.table("http://www.barkhof.uni-bremen.de/~mwindzio/f1erklaere_ihm_matrix3.dat") #  explaining academic issues network
attributes <- read.table("http://www.barkhof.uni-bremen.de/~mwindzio/attribute3.dat", header = TRUE) # actor attributes


#friends <- read.table("fr3.dat") # friendship network
#explain <- read.table("f1erklaere_ihm_matrix3.dat") #  explaining academic issues network
#attributes <- read.table("attribute3.dat", header = TRUE) # actor attributes



# an egdelist is an efficient way to enter network data
# in you empiricas study, just insert existing edges, not in a matrix
# this prevents you from inserting any zeros
# sna has commands to convert and reconvert between different data formats

friend <-as.matrix(friends)

# columns in egdelist: sender, receiver, value (snd rec val)
friends_edge<- as.edgelist.sna(friend)  
friends_edge

# assume that egdelist is the original data
# now covert it into a sociomatrix
friends_mat <- as.sociomatrix.sna(friends_edge)
friends_mat

# we don't need this object, so delete it
rm(friends_mat) # drop in Stata

# our friends matrix from above is still not a network object
# so convert it, because then it is easy to add actor attributes
friend
friends <- network(friend,matrix.type="adjacency",directed=TRUE)
friends

plot(friends)

# create arbitrary id numbers. This could be also an object with
# nodes' names
ids <- c(1:109)
ids


# now merge the actor attributes to the network object with %v%
# important for any kind of actor-related analysis
friends %v% "alter" <- attributes$f2Alter
friends %v% "frau" <- attributes$frau
friends %v% "migrant" <- attributes$mh

# Let's first establish sex-stereotypic colors for the nodes
# frau (female) ==1   
sexColors <- ifelse(attributes$frau==1,yes="hotpink",no="dodgerblue")
plot(friends,vertex.col=sexColors,label.pos=5,vertex.cex=2)

migColors<-ifelse(attributes$mh==1,yes="yellow",no="dodgerblue")
plot(friends,vertex.col=migColors,label.pos=5,vertex.cex=2)

plot(friends,vertex.col=migColors,label=ids,label.pos=5,vertex.cex=2)

# degree centrality
# We begin with the simplest case: degree centrality
degree(friends)  	# Default: total degree
ideg <- degree(friends,cmode="indegree") # in-degree, write into an object 
odeg <- degree(friends,cmode="outdegree") # out-degree, wrtite into an object

ideg
odeg

# what is the mean degree?
mean(degree(friends,cmode="indegree"))


# Betweenness and closeness are also popular measures
bet <- betweenness(friends,gmode="graph")
bet
plot(friends,vertex.cex=(sqrt(bet)/23)+1)
clo <- closeness(friends,gmode="graph") # Geographic closeness
clo   	
plot(friends,vertex.cex=(clo*2)+0.5,vertex.col=migColors)
plot(friends,vertex.cex=(clo*5)+1,vertex.col=migColors,label=ids,label.pos=5)


# centralization of the whole graph ("global" measure), by different measures , Freeman's formula
centralization(friends,degree,cmode="indegree")
centralization(friends,degree,cmode="outdegree")
centralization(friends,closeness)

# Density
gden(friends)  

# Edgewise reciprocity (based on edges present)
# share of mutual ties at all ties realized

grecip(friends, measure="edgewise")

# Transitivity
# share of transitive ties at all ties realized
gtrans(friends)  

# compare our network with a random network
# regarding Transitivity and reciprocity/mutuality
# this is a first motivation of ERGMs
den <- gden(friends)
nnodes <- nrow(as.matrix(friends)) # alternative: network.size(friends)

random_graph<-network(nnodes, directed=TRUE, density=den, seed=1234) # create random digraph
random_graph
plot(random_graph)
plot(friends)
plot(friends, displayisolates=TRUE)

# intractive graph. "Finished" must be pressed to continue  script
plot(friends, vertex.col="white",edge.col="darkgray", label=ids, label.pos=5,label.cex=1, vertex.cex=3,
     interactive = TRUE,usecurve=TRUE, edge.curve=0.01)



# compare reciprocity
grecip(friends, measure="edgewise")
grecip(random_graph, measure="edgewise")

# compare Transitivity
gtrans(friends) 
gtrans(random_graph)

# this is important insight into how emprical social networks
# differ between random networks 


# dyad and triad census
dyad.census(friends)  	# M,A,N counts
dyad.census(random_graph)		# M,A,N counts
triad.census(friends,mode="digraph")	# Directed triad census
triad.census(random_graph,mode="digraph")	# Directed triad census

# TRIAD CENSUS
# 1.) no. of mutual dyads
# 2.) no. of asymmetric dyads
# 3.) no. of null-dyads
# 4. character of further distinction (transitive, cyclic, u=> up, d=>down)

network.edgecount(friends)
network.size(friends)

# alternative network.dyadcount: how many dyads 
network.dyadcount(friends) # 109 * 108
nnodes <- network.size(friends)
nnodes*(nnodes-1)

# density "by hand"
network.edgecount(friends) / ( nnodes*(nnodes-1) )
gden(friends)  


dyad.census(friends)

# friends is network object, but can nevertheless be displayed as a sociomatrix
friends
as.sociomatrix(friends)

# alternative way to plot: gplot, allows layout-algorithms
gplot(friends)  
gplot(friends, mode = "fruchtermanreingold")
gplot(friends, mode = "kamadakawai")
gplot(friends, mode = "kamadakawai",  displaylabels = TRUE)
gplot(friends,  displaylabels = TRUE, mode="mds")

# 3Dplots. beautyful?
gplot3d(friends)
gplot3d(friends, mode = "fruchtermanreingold")
gplot3d(friends, mode = "kamadakawai",  displaylabels = TRUE)


# Get information on geodesics
ideg
odeg

geodist(friends) # inf => node 84 has only some indegree, but not outdegree
friends[84,] # outdegree: clo 84, all rows, display ID 84's outgoing nominations
friends[,84] # indegree: all cols, row 84,  display ID 84's ingoing nominations

# Count cycles of length <=6  => these are a lot, because of combination
kcycle.census(friends,maxlen=6,tabulate.by.vertex=FALSE, mode = "digraph")
kcycle.census(friends,maxlen=6,tabulate.by.vertex=FALSE, mode = "graph")

# Count paths of length <=4 => need some computation time with increasing length
kpath.census(friends,maxlen=4,tabulate.by.vertex=FALSE)

# components
isolates(friends, diag=FALSE)
components(friends) # default is "strong"
components(friends, connected = "weak")   # different comp. are non-reachable groups, here the direction is >not< important
components(friends, connected = "strong") # different comp. are non-reachable groups, here the direction is important

# plot the largest component only
lcomp <- component.largest(friends, connected="strong")
lcomp # just some ">directed< isolates" do not belong to the largest component
gplot(friends[lcomp,lcomp])



# finish: how to write vector with e.g. betweenness centrality
# combine it with your e.g. Stata dataset, perform permutation based regressions etc. 
out <- cbind(ids, bet)
out

write.table(out,file="c:/betweennessCL03.dat", sep=" ", quote = FALSE, row.names = FALSE)
